import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

class XMLDirectory {
	
	private static final String thisProgram = "XMLDirectory";
	private static final String thisVersion = "v1.1";
	
	private static String startPath = ".";
	private static String pattern = "";
	private static boolean recursion = true;
	private static boolean xsltheader = true;
	
	private XMLOutputter outputter;
	private StringFileFilter fileFilter;
	
	private XMLDirectory() {
		// System.out.println( "Beginning" );
		outputter = new XMLOutputter();
		fileFilter = new StringFileFilter( pattern );
		outputter.writePrologue();
		if ( xsltheader ) {
			outputter.writeXSLTHeader();
		}
		outputter.writeOpenTag( "directory_structure" );
		File f = new File( startPath );
		outputter.writeOpenTag( "creation_data" );
		outputter.writeOpenTag( "creator" );
		outputter.writeValue( thisProgram + " " + thisVersion );
		outputter.writeCloseTag( "creator" );
		outputter.writeOpenTag( "start_path" );
		outputter.writeValue( startPath );
		outputter.writeCloseTag( "start_path" );
		try {
			outputter.writeOpenTag( "canonical_start_path" );
			outputter.writeValue( f.getCanonicalPath() );
			outputter.writeCloseTag( "canonical_start_path" );
		} catch ( IOException e ) {
			// ignore this
		}
		outputter.writeOpenTag( "pattern" );
		outputter.writeValue( pattern );
		outputter.writeCloseTag( "pattern" );
		outputter.writeOpenTag( "regex" );
		outputter.writeValue( fileFilter.getRegularExpession() );
		outputter.writeCloseTag( "regex" );
		outputter.writeOpenTag( "recursion" );
		outputter.writeValue( String.valueOf(recursion) );
		outputter.writeCloseTag( "recursion" );
		outputter.writeOpenTag( "xsltheader" );
		outputter.writeValue( String.valueOf(xsltheader) );
		outputter.writeCloseTag( "xsltheader" );
		outputter.writeCloseTag( "creation_data" );
		startCrawling( f );
		outputter.writeCloseTag( "directory_structure" );
		// System.out.println( "End" );
	}
	
	private void startCrawling( File startDirectory ) {
		if ( startDirectory == null || ( ! startDirectory.exists() ) ) {
			// nothing to do
			return;
		}
		if ( startDirectory.isDirectory() ) {
// System.out.println( "--: isDirectory" );		
			outputter.writeOpenTag( "directory" );
			writeFileDetails( startDirectory );
			File[] listOfFiles = startDirectory.listFiles( fileFilter );
			if ( listOfFiles != null ) {
				for ( int i = 0; i < listOfFiles.length; i++ ) {
					if ( ! listOfFiles[i].isHidden() ) {
						crawler( listOfFiles[i] );
					}
				}
			}
			outputter.writeCloseTag( "directory" );
		}
	}
	
	private void crawler( File directoryBeeingInvestigated ) {
		// short name to reduce programming effort
		File dir = directoryBeeingInvestigated;
// System.out.println( "--: " + dir.getAbsolutePath() );		
// System.out.println( "--: " + dir.getName() );		
// System.out.println( "--: exists = " + dir.exists() );		
		if ( dir == null || ( ! dir.exists() ) ) {
			// nothing to do
			return;
		}
		if ( dir.isDirectory() ) {
// System.out.println( "--: isDirectory" );		
			outputter.writeOpenTag( "directory" );
			writeFileDetails( dir );
			if ( recursion ) {
				File[] listOfFiles = dir.listFiles( fileFilter );
				if ( listOfFiles != null ) {
					for ( int i = 0; i < listOfFiles.length; i++ ) {
						// files must be crawled in any case
						// directories are only investigated if recursion is true
						if ( ! listOfFiles[i].isHidden() ) {
							crawler( listOfFiles[i] );
						}
					}
				}
			}
			outputter.writeCloseTag( "directory" );
		} else if ( dir.isFile() ) {
// System.out.println( "--: isFile" );		
			outputter.writeOpenTag( "file" );
			writeFileDetails( dir );
			outputter.writeCloseTag( "file" );
		} else {
// System.out.println( "--: Neither Directory nor File" );		
			System.err.println( "FATAL ERROR: Neither Directory nor File" );
			return;
		}
	}
	
	private void writeFileDetails( File f ) {
		outputter.writeOpenTag( "name" );
		outputter.writeValue( f.getName() );
		outputter.writeCloseTag( "name" );
		if ( f.isFile() ) {
			outputter.writeOpenTag( "size_byte" );
			outputter.writeValue( String.valueOf( f.length() ) );
			outputter.writeCloseTag( "size_byte" );
		}
		outputter.writeOpenTag( "modification_date" );
		Date d = new Date (f.lastModified());
		SimpleDateFormat df;

		outputter.writeOpenTag( "year" );
		df = new SimpleDateFormat ("yyyy");
		outputter.writeValue( df.format(d) );
		outputter.writeCloseTag( "year" );

		outputter.writeOpenTag( "month" );
		df = new SimpleDateFormat ("MM");
		outputter.writeValue( df.format(d) );
		outputter.writeCloseTag( "month" );
		
		outputter.writeOpenTag( "day" );
		df = new SimpleDateFormat ("dd");
		outputter.writeValue( df.format(d) );
		outputter.writeCloseTag( "day" );
		
		outputter.writeOpenTag( "hours" );
		df = new SimpleDateFormat ("HH");
		outputter.writeValue( df.format(d) );
		outputter.writeCloseTag( "hours" );
		
		outputter.writeOpenTag( "minutes" );
		df = new SimpleDateFormat ("mm");
		outputter.writeValue( df.format(d) );
		outputter.writeCloseTag( "minutes" );
		
		outputter.writeOpenTag( "seconds" );
		df = new SimpleDateFormat ("ss");
		outputter.writeValue( df.format(d) );
		outputter.writeCloseTag( "seconds" );
		
		outputter.writeCloseTag( "modification_date" );
	}
	
	private static boolean extractArgs( String argv[] ) {
		// System.out.println( argv.length );
		// no parameter given
		startPath = ".";
		recursion = false;
		xsltheader = true;
		pattern = "*.*";
		if ( argv.length == 0 ) {
			return true;
		}
		// check if help is requested
		if ( argv.length == 1 ) {
			if ( argv[0].toLowerCase().startsWith( "-?" ) ) {
				return false;
			}
			if ( argv[0].toLowerCase().startsWith( "-h" ) ) {
				return false;
			}
		}
		
		String tempPath = "";
		for ( int i = 0; i < argv.length; i++ ) {
			if ( argv[i].toLowerCase().startsWith( "-r" ) ) {
				recursion = true;
			} else if ( argv[i].toLowerCase().startsWith( "-noxslt" ) ) {
				xsltheader = false;
			} else {
				tempPath = new String( argv[i] );
			}
		}
		
		if ( tempPath.length() != 0 ) {
			int lastSep = tempPath.lastIndexOf( File.separatorChar );
			if ( lastSep >= 0 ) {
				startPath = tempPath.substring( 0, lastSep + 1 );
				if ( tempPath.length() > lastSep + 1 ) {
					pattern = tempPath.substring( lastSep+1 );
				}
			} else {
				// tempPath contains only a file pattern
				pattern = tempPath;
			}
		}
		
		return true;
	}
	
	public static void main( String argv[] ) {
		if ( extractArgs( argv ) ) {
			new XMLDirectory(); 
		} else {
			System.out.println( thisProgram + " " + thisVersion );
			System.out.println( "  Usage 1: " + thisProgram + " -?" );
			System.out.println( "           " + thisProgram + " -help" );
			System.out.println( "      Display this help screen." );
			System.out.println();
			System.out.println( "  Usage 2: " + thisProgram + " [-r] [-noxslt] [[path" + File.separator + "]file-pattern]" );
			System.out.println( "      Create XML output for the current directory or the given path" );
			System.out.println( "      with all files matching a given filename pattern" );
			System.out.println( "      -r: recurse all subdirectories" );
			System.out.println( "      -noxslt: avoid generating the '<?xml-stylesheet type=\"text/xsl\" href=\"format.xsl\"?>' line" );
			System.out.println();
		}
	}
	
}
