import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;

class XMLOutputter {
	private static final char intentChar = ' '; 
	private static final int  intentCount = 4; 
	
	private int level = 0;

	public XMLOutputter() {
		// System.out.println( "XMLOutputter initialized" );
	}
	
	private String intent( int intentLevel ) {
		String res = "";
		for ( int i = 0; i < (intentLevel * intentCount); i++ ) {
			res = res + intentChar;
		}
		return res;
	}
	
	private String makeAttribString( Map attribs ) {
		String attrString = "";
		Iterator iKeys = attribs.keySet().iterator();
		while( iKeys.hasNext() ) {
			String sKey = (String) iKeys.next();
			String sVal = (String) attribs.get( sKey );
			attrString += " " + sKey + "=\"" + sVal + "\"";
		}
		return attrString;
	}
	
	public void writePrologue() {
//		System.out.println( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" );
		// This is necessary if there are Umlauts in the file or directory names on Windows Systems
		System.out.println( "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" );
	}
	
	public void writeXSLTHeader() {
		System.out.println( "<?xml-stylesheet type=\"text/xsl\" href=\"format.xsl\"?>" );
	}
	
	public void writeOpenTag( String tag ) {
		writeOpenTag( tag, new HashMap() );
	}
	
	public void writeOpenTag( String tag, Map attribs ) {
		String res = intent( level );
		String attrString = makeAttribString( attribs );
		res = res + "<" + tag + attrString + ">";
		System.out.println( res );
		level++;
	}
	
	public void writeCloseTag( String tag ) {
		level--;
		if ( level < 0 ) {
			level = 0;
		}
		String res = intent( level );
		res = res + "</" + tag + ">";
		System.out.println( res );
	}
	
	public void writeEmptyTag( String tag ) {
		writeEmptyTag( tag, new HashMap() );
	}

	public void writeEmptyTag( String tag, Map attribs ) {
		String res = intent( level );
		String attrString = makeAttribString( attribs );
		res = res + "<" + tag + attrString + "/>";
		System.out.println( res );
	}

	public void writeValue( String value ) {
		String res = intent( level );
		res = res + value;
		System.out.println( res );
	}
	
	// Testing
	private static void executeTest() {
		XMLOutputter out = new XMLOutputter();
		out.writePrologue();
		out.writeOpenTag( "test" );
		HashMap attribs = new HashMap();
		attribs.put( "attrib1", "xxx" );
		attribs.put( "attrib2", "yyy" );
		out.writeOpenTag( "level_2", attribs );
		out.writeOpenTag( "level_3", new HashMap() );
		out.writeValue( "Wert auf Ebene 3" );
		out.writeCloseTag( "level_3" );
		out.writeEmptyTag( "level_3_empty_Tag" );
		attribs = new HashMap();
		attribs.put( "arg1", "abc" );
		out.writeEmptyTag( "level_3_empty_Tag_with_attribs", attribs );
		out.writeCloseTag( "level_2" );
		out.writeOpenTag( "level_2_mit_Ümlaut" );
		out.writeValue( "Das sind viele Umlaute: !§$%/()äöüÄÖÜß" );
		out.writeCloseTag( "level_2_mit_Ümlaut" );
		out.writeCloseTag( "test" );
	}
	
	public static void main( String argv[] ) {
		executeTest();
	}
}
